@@ -6,7 +6,8 @@ COVER=
 UPDATE=0
 PARALLEL='--parallel 0'
 PROFILE="--profile"
-CXX='g++'
+CC="${CC:-gcc}"
+CXX="${CXX:-g++}"
 COVER_DB='cover_db'
 LOCAL_COVERAGE=1
 KEEP_GOING=0
@@ -112,6 +113,7 @@ SELECT_SCRIPT=$SCRIPTS_DIR/select.pm
 CRITERIA_SCRIPT=$SCRIPTS_DIR/criteria.pm
 GITBLAME_SCRIPT=$SCRIPTS_DIR/gitblame.pm
 GITVERSION_SCRIPT=$SCRIPTS_DIR/gitversion.pm
+P4VERSION_SCRIPT=$SCRIPTS_DIR/P4version.pm
 
 # is this git or P4?
 git -C . rev-parse > /dev/null 2>&1
@@ -128,10 +130,10 @@ fi
 # filter out the compiler-generated _GLOBAL__sub_... symbol
 LCOV_BASE="$EXTRA_GCOV_OPTS --branch-coverage $PARALLEL $PROFILE --no-external --ignore unused,unsupported --erase-function .*GLOBAL.*"
 LCOV_OPTS="$LCOV_BASE"
-DIFFCOV_OPTS="--filter line,branch,function --function-coverage --branch-coverage --highlight --demangle-cpp --prefix $PARENT_VERSION $PROFILE "
+DIFFCOV_OPTS="--filter line,branch,function --function-coverage --branch-coverage --demangle-cpp --prefix $PARENT_VERSION $PROFILE "
 
 rm -f test.cpp *.gcno *.gcda a.out *.info *.log *.json diff.txt
-rm -rf select criteria annotate empty unused_src scriptErr scriptFixed epoch inconsistent
+rm -rf select criteria annotate empty unused_src scriptErr scriptFixed epoch inconsistent highlight etc mycache cacheFail expect subset context
 
 if [ "x$COVER" != 'x' ] && [ 0 != $LOCAL_COVERAGE ] ; then
     cover -delete
@@ -154,7 +156,7 @@ ${CXX} --coverage test.cpp
 ./a.out
 
 # old version of gcc has inconsistent line/function data
-IFS='.' read -r -a VER <<< `gcc -dumpversion`
+IFS='.' read -r -a VER <<< `${CC} -dumpversion`
 if [ "${VER[0]}" -lt 5 ] ; then
     # can't get branch coverpoints in 'initial' mode, with ancient GCC
     IGNORE="--ignore usage"
@@ -360,6 +362,23 @@ for arg in "--select-script $SELECT_SCRIPT,--range,0:10" \
     fi
 done
 
+echo genhtml $DIFCOV_OPTS initial.info -o p4err --version-script $P4VERSION_SCRIPT,-x
+$COVER $GENHTML_TOOL $DIFFCOV_OPTS initial.info -o p4err --version-script $P4VERSION_SCRIPT,-x 2>&1 | tee p4err.log
+if [ 0 == ${PIPESTATUS[0]} ] ; then
+    echo "ERROR: genhtml select passed by accident"
+    if [ 0 == $KEEP_GOING ] ; then
+        exit 1
+    fi
+fi
+grep "unable to create callback from" p4err.log
+if [ 0 != $? ] ; then
+    echo "ERROR: missing script message"
+    if [ 0 == $KEEP_GOING ] ; then
+        exit 1
+    fi
+fi
+
+
 echo genhtml $DIFCOV_OPTS initial.info -o select --select-script ./select.sh --rc compute_file_version=1
 $COVER $GENHTML_TOOL $DIFFCOV_OPTS initial.info -o select --select-script ./select.sh  --rc compute_file_version=1 2>&1 | tee select_scr.log
 if [ 0 != ${PIPESTATUS[0]} ] ; then
@@ -383,21 +402,68 @@ if [ 0 != $? ] ; then
 fi
 # and again, as a differential report with annotation
 NOW=`date`
-echo genhtml $DIFCOV_OPTS initial.info -o select --select-script ./select.sh --annotate $ANNOTATE_SCRIPT --baseline-file initial.info
-$COVER $GENHTML_TOOL $DIFFCOV_OPTS initial.info -o select --select-script ./select.sh --annotate $ANNOTATE_SCRIPT --baseline-file initial.info --title 'selectExample' --header-title 'this is the header' --date-bins 1,5,22 --baseline-date "$NOW" --prefix x --no-prefix 2>&1 | tee select_scr.log
+rm -rf mycache
+echo genhtml $DIFCOV_OPTS initial.info -o select --select-script ./select.sh --annotate $ANNOTATE_SCRIPT,--cache,mycache --baseline-file initial.info
+$COVER $GENHTML_TOOL $DIFFCOV_OPTS initial.info -o select --select-script ./select.sh --annotate $ANNOTATE_SCRIPT,--cache,mycache --baseline-file initial.info --title 'selectExample' --header-title 'this is the header' --date-bins 1,5,22 --baseline-date "$NOW" --prefix x --no-prefix 2>&1 | tee select_scr.log
 if [ 0 != ${PIPESTATUS[0]} ] ; then
-    echo "ERROR: genhtml select failed"
+    echo "ERROR: genhtml cache failed"
     if [ 0 == $KEEP_GOING ] ; then
         exit 1
     fi
 fi
+if [ ! -d mycache ] ; then
+    echo "did not create 'mycache'"
+fi
+
+#break the cached data - cause corruption error
+for i in `find mycache -type f` ; do
+    echo $i
+    echo xyz > $i
+done
+# have to ignore version mismatch becaure p4annotate also computes version
+echo genhtml $DIFCOV_OPTS initial.info -o cacheFail --select-script ./select.sh --annotate $ANNOTATE_SCRIPT,--cache,mycache --baseline-file initial.info --ignore version
+$COVER $GENHTML_TOOL $DIFFCOV_OPTS initial.info -o cacheFail --select-script ./select.sh --annotate $ANNOTATE_SCRIPT,--cache,mycache --baseline-file initial.info --title 'selectExample' --header-title 'this is the header' --date-bins 1,5,22 --baseline-date "$NOW" --prefix x --no-prefix --ignore version 2>&1 | tee cacheFail.log
+if [ 0 == ${PIPESTATUS[0]} ] ; then
+    echo "ERROR: genhtml corrupt deserialize failed"
+    if [ 0 == $KEEP_GOING ] ; then
+        exit 1
+    fi
+fi
+
+grep -E "corrupt.*unable to deserialize" cacheFail.log
+if [ 0 != $? ] ; then
+    echo "ERROR: failed to file cache corruption"
+    if [ 0 == $KEEP_GOING ] ; then
+        exit 1
+    fi
+fi
+
+# make cache file unreadable
+find mycache -type f -exec chmod ugo-r {} \;
+echo genhtml $DIFCOV_OPTS initial.info -o cacheFail --select-script ./select.sh --annotate $ANNOTATE_SCRIPT,--cache,mycache --baseline-file initial.info
+$COVER $GENHTML_TOOL $DIFFCOV_OPTS initial.info -o cacheFail --select-script ./select.sh --annotate $ANNOTATE_SCRIPT,--cache,mycache --baseline-file initial.info --title 'selectExample' --header-title 'this is the header' --date-bins 1,5,22 --baseline-date "$NOW" --prefix x --no-prefix 2>&1 | tee cacheFail2.log
+if [ 0 == ${PIPESTATUS[0]} ] ; then
+    echo "ERROR: genhtml unreadable cache failed"
+    if [ 0 == $KEEP_GOING ] ; then
+        exit 1
+    fi
+fi
+
+grep -E "callback.*can't open" cacheFail2.log
+if [ 0 != $? ] ; then
+    echo "ERROR: failed to file cache error"
+    if [ 0 == $KEEP_GOING ] ; then
+        exit 1
+    fi
+fi
+
 
 # differntial report with empty diff file
 touch diff.txt
 echo genhtml $DIFCOV_OPTS initial.info -o empty --diff diff.txt --annotate $ANNOTATE_SCTIPT --baseline-file initial.info
 $COVER $GENHTML_TOOL $DIFFCOV_OPTS initial.info -o empty --diff diff.txt --annotate $ANNOTATE_SCRIPT --baseline-file initial.info 2>&1 | tee empty_diff.log
 if [ 0 == ${PIPESTATUS[0]} ] ; then
-    echo "ERROR: genhtml select failed"
+    echo "ERROR: genhtml did not fail empty diff eheck"
     if [ 0 == $KEEP_GOING ] ; then
         exit 1
     fi
@@ -595,6 +661,89 @@ if [ 0 != $? ] ; then
     fi
 fi
 
+# deprecated messages
+echo genhtml $DIFFCOV_OPTS initial.info -o highlight --highlight
+$COVER $GENHTML_TOOL $DIFFCOV_OPTS initial.info --annotate $ANNOTATE_SCRIPT --highlight -o highlight 2>&1 | tee highlight.log
+if [ 0 == ${PIPESTATUS[0]} ] ; then
+    echo "ERROR: missed decprecated error"
+    if [ 0 == $KEEP_GOING ] ; then
+        exit 1
+    fi
+fi
+grep -E "ERROR: \(deprecated\) .*option .+ has been removed" highlight.log
+if [ 0 != $? ] ; then
+    echo "ERROR: missing highlight message"
+    if [ 0 == $KEEP_GOING ] ; then
+        exit 1
+    fi
+fi
+
+mkdir -p etc
+echo "genhtml_highlight = 1" > etc/lcovrc
+echo genhtml $DIFFCOV_OPTS initial.info -o highlight --config-file LCOV_HOME/etc/lcovrc
+LCOV_HOME=. $COVER $GENHTML_TOOL $DIFFCOV_OPTS initial.info --annotate $ANNOTATE_SCRIPT -o highlight 2>&1 | tee highlight2.log
+if [ 0 != ${PIPESTATUS[0]} ] ; then
+    echo "ERROR: deprecated error was fatal"
+    if [ 0 == $KEEP_GOING ] ; then
+        exit 1
+    fi
+fi
+grep -E "WARNING: \(deprecated\) .+ deprecated and ignored" highlight2.log
+if [ 0 != $? ] ; then
+    echo "ERROR: missing decrecated message"
+    if [ 0 == $KEEP_GOING ] ; then
+        exit 1
+    fi
+fi
+
+for err in "--rc truncate_owner_table=top,x" "--rc owner_table_entries=abc" "--rc owner_table_entries=-1" ; do
+    echo genhtml $DIFCOV_OPTS initial.info -o subset --annotate $ANNOTATE_SCRIPT --baseline-file initial.info --show-owners
+    $COVER $GENHTML_TOOL $DIFFCOV_OPTS initial.info -o subset --annotate $ANNOTATE_SCRIPT --baseline-file initial.info --title 'subset' --header-title 'this is the header' --date-bins 1,5,22 --baseline-date "$NOW" --prefix x --no-prefix $err --show-owners
+    if [ 0 == $? ] ; then
+        echo "ERROR: genhtml $err unexpectedly passed"
+        if [ 0 == $KEEP_GOING ] ; then
+            exit 1
+        fi
+    fi
+done
+
+
+# test error checks for --expect-message-count expressions
+for expr in "malformed" "noSuchMsg:%C<5" "inconsistent:%c<5" 'inconsistent:%C<$x' 'inconsistent:0,inconsistent:2' ; do
+
+    $COVER $GENHTML_TOOL $DIFFCOV_OPTS initial.info -o expect --annotate $ANNOTATE_SCRIPT --baseline-file initial.info --title 'subset' --header-title 'this is the header' --date-bins 1,5,22 --baseline-date "$NOW" --prefix x --no-prefix --expect-message $expr --show-owners
+    if [ 0 == $? ] ; then
+        echo "ERROR: genhtml $expr unexpectedly passed"
+        if [ 0 == $KEEP_GOING ] ; then
+            exit 1
+        fi
+    fi
+
+    $COVER $GENHTML_TOOL $DIFFCOV_OPTS initial.info -o expect --annotate $ANNOTATE_SCRIPT --baseline-file initial.info --title 'subset' --header-title 'this is the header' --date-bins 1,5,22 --baseline-date "$NOW" --prefix x --no-prefix --expect-message $expr --show-owners --ignore usage
+    if [ 0 != $? ] ; then
+        echo "ERROR: genhtml $expr with ignore unexpectedly failed"
+        if [ 0 == $KEEP_GOING ] ; then
+            exit 1
+        fi
+    fi
+done
+
+# slightly more complicated case...hack a bit so that 'expect' eval fails
+#  in summary callback
+$COVER $GENHTML_TOOL $DIFFCOV_OPTS initial.info -o context --annotate $ANNOTATE_SCRIPT --baseline-file initial.info --title 'context' --header-title 'this is the header' --date-bins 1,5,22 --baseline-date "$NOW" --prefix x --no-prefix --context-script ./MsgContext.pm --expect-message 'usage:MsgContext::test(%C)' --show-owners --ignore callback 2>&1 | tee expect.log
+if [ 0 != ${PIPESTATUS[0]} ] ; then
+    echo "ERROR: genhtml context with ignore unexpectedly failed"
+    if [ 0 == $KEEP_GOING ] ; then
+        exit 1
+    fi
+fi
+grep -E "WARNING: .*callback.* evaluation of .+ failed" expect.log
+if [ 0 != $? ] ; then
+    echo "ERROR: didn't find expected callback message"
+    if [ 0 == $KEEP_GOING ] ; then
+        exit 1
+    fi
+fi
 
 echo "Tests passed"
 
